#define TRACE
using System;
using System.Runtime.InteropServices;
using CustomResolutionsTypes;
using System.Linq;
using System.Diagnostics;

namespace Change
{
    [ComVisible(true)]
    [Guid("7ddb810f-928c-46a7-af70-afb0850b2388")]
    [ClassInterface(ClassInterfaceType.None)]
    public class Plugin : ICustomResolutionPlugin, ICustomPluginFormatParams, ICustomResolutionStyles
    {
        #region Ctor
        public Plugin()
        {

        }
        #endregion

        #region ICustomResolutionPlugin
        public String Name
        {
            get
            {
                return "Chg";
            }
        }

        public String Guid
        {
            get
            {
                return "3bfeab33-d603-4ea6-8683-d791dd97c280";
            }
        }

        public String Description
        {
            get
            {
                return "Prince change tick";
            }
        }

        public String Vendor
        {
            get
            {
                return "Wzero";
            }
        }

        #region Properties
        private double m_open;
        private double m_high;
        private double m_low;
        private double m_close;
        private long m_Volume = 0;
        private long m_UpVolume = 0;
        private long m_DownVolume = 0;
        private double m_PointValue = 0.0001;
        private long m_MinMovement = 1;
        #endregion

        public void Init(IBaseOptions baseOptions, IParams customParams)
        {
            if (baseOptions != null)
            {
                m_PointValue = baseOptions.PointValue;
                m_MinMovement = baseOptions.MinMovement;
            }

            Trace.TraceInformation(string.Format("Init {0}: PointValue={1}, MinMovement={2}",
                ToString(), m_PointValue, m_MinMovement));
        }

        public void OnData(ICustomBar Bar, Int64 time_in_ticks, Int32 tickId, double open, double high, double low, double close, long volumeAdded, long upVolumeAdded, long downVolumeAdded, ECustomBarTrendType trend, bool isBarClose)
        {
            if (m_close != 0 && close != m_close)
            {
                Bar.CloseBar();

                m_open = 0;
                m_high = 0;
                m_low = 0;
                m_close = 0;
                m_Volume = 0;
                m_UpVolume = 0;
                m_DownVolume = 0;
            }

            if (m_open == 0)
            {
                m_open = open;
                m_high = high;
                m_low = low;
                m_close = close;
            }
            else
            {
                m_close = close;
                m_high = m_close > m_high ? m_close : m_high;
                m_low = m_close < m_low ? m_close : m_low;
            }

            m_Volume += volumeAdded;
            m_UpVolume += upVolumeAdded;
            m_DownVolume += downVolumeAdded;

            Bar.UpdateBar(time_in_ticks, tickId, m_open, m_high, m_low, m_close, m_Volume, m_UpVolume, m_DownVolume, m_close >= m_open ? ECustomBarTrendType.TrendUp : ECustomBarTrendType.TrendDown, true, true);
        }

        public void Reset()
        {
            m_open = 0;
            m_high = 0;
            m_low = 0;
            m_close = 0;
            m_Volume = 0;
            m_UpVolume = 0;
            m_DownVolume = 0;
        }
        #endregion

        #region ICustomPluginFormatParams
        public void FormatParams(IParams customParams, IPriceScale priceScale, out string formattedParams)
        {
            formattedParams = Name;
        }
        #endregion

        #region ICustomResolutionStyles
        public Int32 StyleCount
        {
            get
            {
                return m_Styles.Length;
            }
        }
        public EStyleType GetStyle(Int32 Idx)
        {
            return m_Styles[Idx];
        }

        private EStyleType[] m_Styles = new EStyleType[] { EStyleType.LineOnClose };
        #endregion
    }
}

